# Fichier : c.gp
# Version : 3
# Date : 19/10/23
# Courbes comparatives par mois et par jour entre deux années
# Usage : gnuplot -e "file1='2021'; file2='2022'" c.gp

set datafile separator "/;"
set decimalsign locale
set style data lines
set xrange [1:]
set grid ytics
set y2tics
set terminal pdfcairo size 29.7 cm, 21 cm
set output sprintf("%s-%s_comparatif.pdf", file1, file2)

file1_csv = sprintf("%s_mois.csv", file1)
file2_csv = sprintf("%s_mois.csv", file2)
climat1_csv = sprintf("%s_climat.csv", file1)
climat2_csv = sprintf("%s_climat.csv", file2)

stats file1_csv using 3 name "HP1"
stats file1_csv using 4 name "HCN1"
stats file1_csv using 5 name "HCM1"
stats file1_csv using 6 name "HCT1"
stats file1_csv using 7 name "T1"

stats file2_csv using 3 name "HP2"
stats file2_csv using 4 name "HCN2"
stats file2_csv using 5 name "HCM2"
stats file2_csv using 6 name "HCT2"
stats file2_csv using 7 name "T2"

set title sprintf("%s - %s Comparatif consommation totale, par mois", file1, file2)
plot file1_csv using 1:7 title sprintf("%s  (%d kWh)", file1, T1_sum) lc 7,\
     file2_csv using 1:7 title sprintf("%s  (%d kWh)", file2, T2_sum) lc 8, \
     climat1_csv using 1:3 title sprintf("%s  (°C)", file1) lc 7 dt (20, 6) axes x1y2, \
     climat2_csv using 1:3 title sprintf("%s  (°C)", file2) lc 8 dt (20, 6) axes x1y2

set title sprintf("%s - %s Comparatif heures pleines, par mois", file1, file2)
plot file1_csv using 1:3 title sprintf("%s  (%d kWh)", file1, HP1_sum) lc 7,\
     file2_csv using 1:3 title sprintf("%s  (%d kWh)", file2, HP2_sum) lc 8, \
     climat1_csv using 1:3 title sprintf("%s  (°C)", file1) lc 7 dt (20, 6) axes x1y2, \
     climat2_csv using 1:3 title sprintf("%s  (°C)", file2) lc 8 dt (20, 6) axes x1y2

set title sprintf("%s - %s Comparatif heures creuses totales, par mois", file1, file2)
plot file1_csv using 1:6 title sprintf("%s  (%d kWh)", file1, HCT1_sum) lc 7,\
     file2_csv using 1:6 title sprintf("%s  (%d kWh)", file2, HCT2_sum) lc 8, \
     climat1_csv using 1:3 title sprintf("%s  (°C)", file1) lc 7 dt (20, 6) axes x1y2, \
     climat2_csv using 1:3 title sprintf("%s  (°C)", file2) lc 8 dt (20, 6) axes x1y2

set title sprintf("%s - %s Comparatif heures creuses nuit, par mois", file1, file2)
plot file1_csv using 1:4 title sprintf("%s  (%d kWh)", file1, HCN1_sum) lc 7,\
     file2_csv using 1:4 title sprintf("%s  (%d kWh)", file2, HCN2_sum) lc 8, \
     climat1_csv using 1:3 title sprintf("%s  (°C)", file1) lc 7 dt (20, 6) axes x1y2, \
     climat2_csv using 1:3 title sprintf("%s  (°C)", file2) lc 8 dt (20, 6) axes x1y2

set title sprintf("%s - %s Comparatif heures creuses jour, par mois", file1, file2)
plot file1_csv using 1:5 title sprintf("%s  (%d kWh)", file1, HCM1_sum) lc 7,\
     file2_csv using 1:5 title sprintf("%s  (%d kWh)", file2, HCM2_sum) lc 8, \
     climat1_csv using 1:3 title sprintf("%s  (°C)", file1) lc 7 dt (20, 6) axes x1y2, \
     climat2_csv using 1:3 title sprintf("%s  (°C)", file2) lc 8 dt (20, 6) axes x1y2

set xdata time
set timefmt "%d/%m/%Y"
set format x "%d/%m/%Y"
set xtics rotate out
unset y2tics
unset xrange

file1_csv = sprintf("%s_jour.csv", file1)
file2_csv = sprintf("%s_jour.csv", file2)

set title sprintf("%s - %s Comparatif consommation totale, par jour", file1, file2)
# Note : si on fait apparaître l'axe x des deux courbes, on voit un léger décalage.
unset multiplot
set multiplot
set lmargin at screen 0.05
set rmargin at screen 0.98
set tmargin at screen 0.95
set bmargin at screen 0.1
set yrange [0:120]
set xrange [sprintf("01/01/%s", file1):sprintf("31/12/%s", file1)]
plot file1_csv using 1:8 title sprintf("%s  (%d kWh)", file1, T1_sum) at 0.9, 0.9 lc 7
unset xtics
unset ytics
set xrange [sprintf("01/01/%s", file2):sprintf("31/12/%s", file2)]
plot file2_csv using 1:8 title sprintf("%s  (%d kWh)", file2, T2_sum) at 0.9, 0.85 lc 8, \

set xtics
set ytics
set title sprintf("%s - %s Comparatif heures pleines, par jour", file1, file2)
# Note : si on fait apparaître l'axe x des deux courbes, on voit un léger décalage.
unset multiplot
set multiplot
set lmargin at screen 0.05
set rmargin at screen 0.98
set tmargin at screen 0.95
set bmargin at screen 0.1
set yrange [0:40]
set xrange [sprintf("01/01/%s", file1):sprintf("31/12/%s", file1)]
plot file1_csv using 1:4 title sprintf("%s  (%d kWh)", file1, HP1_sum) at 0.9, 0.9 lc 7
unset xtics
unset ytics
set xrange [sprintf("01/01/%s", file2):sprintf("31/12/%s", file2)]
plot file2_csv using 1:4 title sprintf("%s  (%d kWh)", file2, HP2_sum) at 0.9, 0.85 lc 8, \

set xtics
set ytics
set title sprintf("%s - %s Comparatif heures creuses totales, par jour", file1, file2)
# Note : si on fait apparaître l'axe x des deux courbes, on voit un léger décalage.
unset multiplot
set multiplot
set lmargin at screen 0.05
set rmargin at screen 0.98
set tmargin at screen 0.95
set bmargin at screen 0.1
set yrange [0:80]
set xrange [sprintf("01/01/%s", file1):sprintf("31/12/%s", file1)]
plot file1_csv using 1:7 title sprintf("%s  (%d kWh)", file1, HCT1_sum) at 0.9, 0.9 lc 7
unset xtics
unset ytics
set xrange [sprintf("01/01/%s", file2):sprintf("31/12/%s", file2)]
plot file2_csv using 1:7 title sprintf("%s  (%d kWh)", file2, HCT2_sum) at 0.9, 0.85 lc 8, \

set xtics
set ytics
set title sprintf("%s - %s Comparatif heures creuses nuit, par jour", file1, file2)
# Note : si on fait apparaître l'axe x des deux courbes, on voit un léger décalage.
unset multiplot
set multiplot
set lmargin at screen 0.05
set rmargin at screen 0.98
set tmargin at screen 0.95
set bmargin at screen 0.1
set yrange [0:60]
set xrange [sprintf("01/01/%s", file1):sprintf("31/12/%s", file1)]
plot file1_csv using 1:5 title sprintf("%s  (%d kWh)", file1, HCN1_sum) at 0.9, 0.9 lc 7
unset xtics
unset ytics
set xrange [sprintf("01/01/%s", file2):sprintf("31/12/%s", file2)]
plot file2_csv using 1:5 title sprintf("%s  (%d kWh)", file2, HCN2_sum) at 0.9, 0.85 lc 8, \

set xtics
set ytics
set title sprintf("%s - %s Comparatif heures creuses jour, par jour", file1, file2)
# Note : si on fait apparaître l'axe x des deux courbes, on voit un léger décalage.
unset multiplot
set multiplot
set lmargin at screen 0.05
set rmargin at screen 0.98
set tmargin at screen 0.95
set bmargin at screen 0.1
set yrange [0:20]
set xrange [sprintf("01/01/%s", file1):sprintf("31/12/%s", file1)]
plot file1_csv using 1:6 title sprintf("%s  (%d kWh)", file1, HCM1_sum) at 0.9, 0.9 lc 7
unset xtics
unset ytics
set xrange [sprintf("01/01/%s", file2):sprintf("31/12/%s", file2)]
plot file2_csv using 1:6 title sprintf("%s  (%d kWh)", file2, HCM2_sum) at 0.9, 0.85 lc 8
