/*
Program: hc
Version: 2.0
Date: 28/08/23

=======================================================================

Copyright 2021-2023 Gérard Leclercq

Ce programme est un logiciel libre. Vous pouvez le redistribuer et/ou le modifier suivant les conditions de la licence publique générale GNU publiée par la Free Software Foundation, soit à la version 3 de la licence, ou, à votre convenance, à n'importe quelle version ultérieure.

Ce programme est distribué dans la pensée qu'il peut être utile, mais SANS AUCUNE GARANTIE, sans même la garantie implicite de QUALITÉ MARCHANDE ou d'ADÉQUATION À UN USAGE PARTICULIER. Voir les conditions de la licence publique générale GNU pour plus de détails.

Vous devriez avoir une copie de la licence publique générale GNU dans ce fichier zip. Sinon, voir www.gnu.org/licenses.

=======================================================================

Notes :

- Le fichier csv d'Enedis est codé en UTF-8 BOM. Un marqueur d'ordonnancement (appelé BOM pour Byte Order Mark) est codé en début de fichier à l'aide du caractère U+FEFF (https://fr.wikipedia.org/wiki/UTF-8).

- Dans les fichiers Enedis, la puissance moyenne indiquée sur une ligne correspond à l'intervalle de temps finissant à l'heure indiquée sur cette même ligne et commençant à l'heure indiquée sur la ligne précédente. Pour la première ligne donnant une puissance, l'intervalle de temps commence à l'heure 0.

- En 2022, la puissance moyenne est un nombre toujours pair !!!

=======================================================================

*/

/* Les deux lignes suivantes sont nécessaires pour la fonction asprintf. */
#define __STDC_WANT_LIB_EXT2__ 1
#define _GNU_SOURCE

#include	<stdio.h>
#include	<string.h>
#include	<stdlib.h>
#include	<math.h>
#include    <locale.h>

#define	FILE_LINE_MAX_SIZE 1000

float hc_nuit_debut_mn;
float hc_nuit_fin_mn;
float hc_jour_debut_mn;
float hc_jour_fin_mn;



int hhmn_to_mm(hh, mm)
char *hh;
char *mm;
{
    return atoi(hh)*60 + atoi(mm);
}



int conversion_heures_creuses_en_minutes(argv)
char	*argv[];
{
    char    *heure;
    char    *minute;
    
	printf("Heure Creuse nuit debut %s\n", argv[2]);
	printf("Heure Creuse nuit fin   %s\n", argv[3]);
	printf("Heure Creuse jour debut %s\n", argv[4]);
	printf("Heure Creuse jour fin   %s\n\n", argv[5]);
    
    if ((strlen(argv[2]) < 5) || (strlen(argv[3]) < 5) || (strlen(argv[4]) < 5) || (strlen(argv[5]) < 5))  {
        printf("\nErreur : l'heure creuse doit etre sous la forme xxhxx (exemple : 01h30)\n");
		exit(EXIT_FAILURE);
    }
    
	heure = strdup(argv[2]);
	heure[2] = '\0';
	minute = strdup(argv[2]);
	minute[0] = minute[3];
	minute[1] = minute[4];
	minute[2] = '\0';
    hc_nuit_debut_mn = atof(heure) * 60 + atof(minute);
    
	heure = strdup(argv[3]);
	heure[2] = '\0';
	minute = strdup(argv[3]);
	minute[0] = minute[3];
	minute[1] = minute[4];
	minute[2] = '\0';
    hc_nuit_fin_mn = atof(heure) * 60 + atof(minute);
    
	heure = strdup(argv[4]);
	heure[2] = '\0';
	minute = strdup(argv[4]);
	minute[0] = minute[3];
	minute[1] = minute[4];
	minute[2] = '\0';
    hc_jour_debut_mn = atof(heure) * 60 + atof(minute);
    
	heure = strdup(argv[5]);
	heure[2] = '\0';
	minute = strdup(argv[5]);
	minute[0] = minute[3];
	minute[1] = minute[4];
	minute[2] = '\0';
    hc_jour_fin_mn = atof(heure) * 60 + atof(minute);

    
}



int nb_argument(nb_arg)
int nb_arg;
{
	if (nb_arg < 6) {
		printf("\nUsage: hc  fichier_Enedis.csv  HC_nuit_debut  HC_nuit_fin  HC_jour_debut  HC_jour_fin\n\n");
		printf("Format des Heures Creuses : xxhxx\n\n");
		printf("Exemples :\n");
		printf("hc 2022.csv 01h00 07h30 12h30 14h00\n");
		printf("hc 2022.csv 00h00 06h00 22h00 24h00\n");
		exit(EXIT_FAILURE);
	}
}



int main(argc, argv)
int 	argc;
char	*argv[];
{
    char    *aaaamm, *aaaamm1 ; /* Mois et année au format Enedis. */
    char    *aaaammjj, *aaaammjj1 ; /* Date au format Enedis. */
    char    *annee, *mois, *jour, *annee1, *mois1, *jour1 ; /* Exemple : 2022, 09, 01. */
	char	*f_enedis; /* Nom du fichier Enedis sans l'extension ".csv". */
	char	*f_enedis_reformate_csv, *f_jour_csv, *f_mois_csv; /* Nom des fichiers avec leur extension. */
	FILE	*fp_enedis_csv; /* Fichier csv Enedis. */
    FILE    *fp_enedis_reformate_csv; /* Fichier csv Enedis reformaté pour être plus facilement exploitable par un tableur. */
    FILE    *fp_jour_csv; /* Fichier csv des consommations journalières */;
    FILE    *fp_mois_csv; /* Fichier csv des consommations mensuelles */;
    long int    fpos_enedis; /* Position dans la lecture du fichier. */
    float    hc_jour, hc_nuit, hp, hc_jour1, hc_nuit1, hp1; /* Consommations journalière en Wh : heures creuses midi, heures creuses nuit, heures pleines. */
    float    hc_jour_m, hc_nuit_m, hp_m; /* Consommations mensuelle en Wh : heures creuses midi, heures creuses nuit, heures pleines. */
    char    *heure;
	char	ligne[FILE_LINE_MAX_SIZE], *ligne1;
    char    *minute; /* Minutes des heures, de 00 à 60. */
    int     minutes, minutes1; /* Nombre de minutes depuis l'heure 0, de 0 à 24*60. */
    float     puissance; /* Puissance moyenne en watt du temps de la ligne précédente au temps de la ligne actuelle. */
	
    nb_argument(argc);
    
    conversion_heures_creuses_en_minutes(argv);
	
	f_enedis = strdup(argv[1]);
	f_enedis[strlen(argv[1]) - 4] = '\0';
    asprintf(&f_enedis_reformate_csv, "%s%s", f_enedis, "_reformate.csv");
    asprintf(&f_jour_csv, "%s%s", f_enedis, "_jour.csv");
    asprintf(&f_mois_csv, "%s%s", f_enedis, "_mois.csv");
    
	fp_enedis_csv = fopen(argv[1], "rb");
	fp_enedis_reformate_csv = fopen(f_enedis_reformate_csv, "w");
	fp_jour_csv = fopen(f_jour_csv, "w");
	fp_mois_csv = fopen(f_mois_csv, "w");
	
    minutes1 = 0;
    
    /* Entête. */
	fgets(ligne, FILE_LINE_MAX_SIZE, fp_enedis_csv);
    ligne1 = strdup(ligne);
    printf("%s", ligne1 + 3); /* + 3 cat encodage BOM du UTF-8. */
    
	fgets(ligne, FILE_LINE_MAX_SIZE, fp_enedis_csv);
    ligne1 = strdup(ligne);
    printf("%s", ligne1);
    
	fgets(ligne, FILE_LINE_MAX_SIZE, fp_enedis_csv);
    ligne1 = strdup(ligne);
    printf("%s", ligne1);
    
    fpos_enedis = ftell(fp_enedis_csv);
    
    /* Valeurs. */
	fgets(ligne, FILE_LINE_MAX_SIZE, fp_enedis_csv);
    printf("%s\n", ligne);
    aaaammjj1 = strdup(ligne);
    aaaammjj1[10] = '\0';
    aaaamm1 = strdup(aaaammjj1);
    aaaamm1[7] = '\0';
    annee1 = strdup(aaaammjj1);
    annee1[4] = '\0';
    mois1 = strdup(aaaammjj1 + 5);
    mois1[2] = '\0';
    jour1 = strdup(aaaammjj1 + 8);
    jour1[2] = '\0';
    heure = strdup(ligne) + 11;
    heure[2] = '\0';
    minute = strdup(ligne) + 14;
    minute[2] = '\0';
    puissance = atoi(ligne + 26);
    minutes = hhmn_to_mm(heure, minute);
    // fseek(fp_enedis_csv, fpos_enedis, SEEK_SET);
    
    hc_jour = 0;
    hc_nuit = 0;
    hp = 0;
    
    hc_jour_m = 0;
    hc_nuit_m = 0;
    hp_m = 0;
    
    setlocale(LC_NUMERIC, "FRENCH");
    
    printf("Date et minutes de depart : %s %5d\n", aaaammjj1, minutes);
    
    fprintf(fp_enedis_reformate_csv, "#Les puissance et consommations d'une ligne sont données jusqu'à l'heure/minute indiquée sur la même ligne\n");
    fprintf(fp_enedis_reformate_csv, "#Date;heure;minute;Puissance moyenne (W);Heures pleines (Wh);Heures creuses nuit (Wh);Heures creuses jour (Wh)\n");
	fprintf(fp_enedis_reformate_csv, "#Heure Creuse nuit debut %s\n", argv[2]);
	fprintf(fp_enedis_reformate_csv, "#Heure Creuse nuit fin   %s\n", argv[3]);
	fprintf(fp_enedis_reformate_csv, "#Heure Creuse jour debut %s\n", argv[4]);
	fprintf(fp_enedis_reformate_csv, "#Heure Creuse jour fin   %s\n", argv[5]);
    
    fprintf(fp_jour_csv, "#Date;Heures pleines (kWh);Heures creuses nuit (kWh);Heures creuses jour (kWh);Heures creuses total (kWh);Total (kWh)\n");
    
    fprintf(fp_mois_csv, "#Mois;Heures pleines (kWh);Heures creuses nuit (kWh);Heures creuses jour (kWh);Heures creuses total (kWh);Total (kWh)\n");

	fgets(ligne, FILE_LINE_MAX_SIZE, fp_enedis_csv);
	
	do {
        if (strcmp(ligne, ligne1)) { /* Il y a parfois des lignes en double ! */
            aaaammjj = strdup(ligne);
            aaaammjj[10] = '\0';
            aaaamm = strdup(aaaammjj);
            aaaamm[7] = '\0';
            annee = strdup(aaaammjj);
            annee[4] = '\0';
            mois = strdup(aaaammjj + 5);
            mois[2] = '\0';
            jour = strdup(aaaammjj + 8);
            jour[2] = '\0';
            heure = strdup(ligne) + 11;
            heure[2] = '\0';
            minute = strdup(ligne) + 14;
            minute[2] = '\0';
            puissance = atoi(ligne + 26);
            
            minutes = hhmn_to_mm(heure, minute);
            if (minutes == 0) {
                minutes = 1440; /* 24h00 */
            }
                
            if (( minutes <= hc_nuit_debut_mn ) ||
            (( minutes > hc_nuit_fin_mn ) && ( minutes <= hc_jour_debut_mn )) ||
                ( minutes > hc_jour_fin_mn )) {
                hp1 = puissance * (minutes - minutes1) / 60;
                hp += hp1;
                }
            else if (( minutes > hc_nuit_debut_mn ) && ( minutes <= hc_nuit_fin_mn )) {
                hc_nuit1 = puissance * (minutes - minutes1) / 60;
                hc_nuit += hc_nuit1;
            }
            else if (( minutes > hc_jour_debut_mn ) && ( minutes <= hc_jour_fin_mn )) {
                hc_jour1 = puissance * (minutes - minutes1) / 60;
                hc_jour += hc_jour1;
            }
            fprintf(fp_enedis_reformate_csv, "%s/%s/%s;%s;%s;%.0f;%.0f;%.0f;%.0f\n", jour, mois, annee, heure, minute, puissance, hp1, hc_nuit1, hc_jour1);
            hp1 = 0;
            hc_nuit1= 0;
            hc_jour1 = 0;
            
            minutes1 = minutes;
            if (minutes == 1440) {
                minutes1 = 0;
            }
            
            /* Changement de jour. */
            if (strcmp(aaaammjj, aaaammjj1) && (minutes == 1440)) {
                fprintf(fp_jour_csv, "%s/%s/%s;%.3f;%.3f;%.3f;%.3f;%.3f\n", jour1, mois1, annee1, hp/1000., hc_nuit/1000., hc_jour/1000., (hc_nuit + hc_jour)/1000., (hp + hc_nuit + hc_jour)/1000.);
        
                hc_jour_m += hc_jour;
                hc_nuit_m += hc_nuit;
                hp_m += hp;
                
                hc_jour = 0;
                hc_nuit = 0;
                hp = 0;
            
                /* Changement de mois. */
                if (strcmp(aaaamm, aaaamm1) && (minutes == 1440)) {
                    fprintf(fp_mois_csv, "%s/%s;%.3f;%.3f;%.3f;%.3f;%.3f\n", mois1, annee1, hp_m/1000., hc_nuit_m/1000., hc_jour_m/1000., (hc_nuit_m + hc_jour_m)/1000., (hp_m + hc_nuit_m + hc_jour_m)/1000.);
            
                    hc_jour_m = 0;
                    hc_nuit_m = 0;
                    hp_m = 0;
                    
                    strcpy(aaaamm1, aaaamm);
                }
                
                strcpy(aaaammjj1, aaaammjj);
                strcpy(jour1, jour);
                strcpy(mois1, mois);
                strcpy(annee1, annee);
            }
        }
        
        strcpy(ligne1, ligne);

	} while (fgets(ligne, FILE_LINE_MAX_SIZE, fp_enedis_csv) != NULL);
		
    if ((hp_m != 0.) && (hc_nuit_m != 0.) &&  (hc_jour_m != 0.))
        fprintf(fp_mois_csv, "%s/%s;%.3f;%.3f;%.3f;%.3f;%.3f\n", mois1, annee1, hp_m/1000., hc_nuit_m/1000., hc_jour_m/1000., (hc_nuit_m + hc_jour_m)/1000., (hp_m + hc_nuit_m + hc_jour_m)/1000.);
    printf("Date et minutes de fin    : %s %5d\n", aaaammjj1, minutes);
            
	fclose(fp_enedis_csv);
	fclose(fp_enedis_reformate_csv);
	fclose(fp_jour_csv);
	fclose(fp_mois_csv);

}
